@@ -86,7 +86,7 @@ Parameters::Parameters(
 
     // Set validated flag and instantiate Version
     validated        ( false ),
-    version          ( 1, 7, 0, "2020-11-7" )
+    version          ( 1, 7, 1, "2020-11-29" )
 {
     // Constructor code
     if ( method != Method::None ) {
@@ -179,6 +179,86 @@ void Parameters::Validate() {
                 i++;
             }
         }
+
+        // If disjointLibrary create vector of disallowed library rows
+        if ( disjointLibrary ) {
+            int maxLibrary_i = (int) library.back();
+            int minLibrary_i = (int) library.front();
+            int allLibSize   = maxLibrary_i - minLibrary_i + 1;
+
+            // A "full" library with no missing values, for set difference
+            std::vector< size_t > allLibrary( allLibSize );
+
+            // Fill with indices 0, 1, ... allLibSize - 1
+            i = 0;
+            for ( int lib_i = minLibrary_i; lib_i <= maxLibrary_i; lib_i++ ) {
+                allLibrary[ i ] = lib_i;
+                i++;
+            }
+
+            // Find library rows that are "gaps", not in library
+            disjointLibraryRows = std::vector< size_t > ( allLibSize );
+            std::vector< size_t >::iterator li;
+            li = std::set_difference ( allLibrary.begin(), allLibrary.end(),
+                                       library.begin(),    library.end(),
+                                       disjointLibraryRows.begin());
+
+            disjointLibraryRows.resize( li - disjointLibraryRows.begin() );
+
+            // Add rows that span gaps by virture of embedding
+            int shift = tau * ( E - 1 ); // default tau = -1
+            std::vector< size_t > disjointLibraryRowsCopy( disjointLibraryRows );
+            
+            for ( auto disjointLibRow : disjointLibraryRowsCopy ) {
+                int thisRow = (int) disjointLibRow;
+                int newRow  = embedded ? thisRow : thisRow - shift;
+
+                if ( newRow > 0 ) {
+                    li = std::find( disjointLibraryRowsCopy.begin(),
+                                    disjointLibraryRowsCopy.end(), newRow );
+                    if ( li == disjointLibraryRowsCopy.end() ) {
+                        // newRow not in disjointLibraryRows
+                        disjointLibraryRows.push_back( (size_t) newRow );
+                    }
+                }
+            }
+
+            // Remove from library
+            std::vector< size_t >::iterator dli;
+            for ( dli  = disjointLibraryRows.begin();
+                  dli != disjointLibraryRows.end(); dli++ ) {
+                
+                li = std::find( library.begin(), library.end(), *dli );
+                if ( li != library.end() ) {
+                    library.erase( li );
+                }   
+            }
+        } // disjointLibrary
+
+        if ( method == Method::Simplex or
+             method == Method::SMap or method == Method::CCM ) {
+
+            // Validate lib: E, tau, Tp combination
+            int vectorStart  = std::max( (E - 1) * tau, 0 );
+            vectorStart      = std::max( vectorStart, Tp );
+            int vectorEnd    = std::min( (E - 1) * tau, Tp );
+            vectorEnd        = std::min( vectorEnd, 0 );
+            int vectorLength = std::abs( vectorStart - vectorEnd ) + 1;
+
+            int maxLibrarySegment = 0;
+            for ( auto thisPair : libPairs ) {
+                int libPairSpan = thisPair.second - thisPair.first + 1;
+                maxLibrarySegment = std::max( maxLibrarySegment, libPairSpan );
+            }
+
+            if ( vectorLength > maxLibrarySegment ) {
+                std::stringstream errMsg;
+                errMsg << "Parameters::Validate(): Combination of E = "
+                       << E << " Tp = " << Tp << " tau = " << tau
+                       << " is invalid.\n";
+                throw std::runtime_error( errMsg.str() );
+            }
+        }
     }
 
     //--------------------------------------------------------------
@@ -275,7 +355,7 @@ void Parameters::Validate() {
     // If columns are purely integer, set vector<size_t> columnIndex
     // Otherwise fill in vector<string> columnNames
     //--------------------------------------------------------------
-   if ( columns_str.size() ) {
+    if ( columns_str.size() ) {
 
         std::vector<std::string> columns_vec = SplitString( columns_str,
                                                             " \t,\n" );
@@ -410,36 +490,36 @@ void Parameters::Validate() {
     }
 
     //--------------------------------------------------------------------
-    // Simplex and knn not specified: not embedded : knn set to E+1
-    //                                    embedded : knn set to size( columns )
+    // Simplex: knn not specified: knn set to E+1
+    //    embedded true : E set to number columns
     //--------------------------------------------------------------------
     if ( method == Method::Simplex or method == Method::CCM ) {
-        if ( knn < 1 ) {
-            if ( not embedded ) {
-                knn = E + 1;
-                if ( verbose ) {
-                    std::stringstream msg;
-                    msg << "Parameters::Validate(): Set knn = " << knn
-                        << " (E+1) for Simplex. " << std::endl;
-                    std::cout << msg.str();
+
+        // embedded = true: Set E to number of columns if not already set
+        if ( embedded ) {
+            if ( columnIndex.size() ) {
+                if ( E != (int) columnIndex.size() ) {
+                    E = columnIndex.size();
                 }
             }
-            else { // embedded = true
-                if ( columnIndex.size() ) {
-                    knn = columnIndex.size() + 1;
-                }
-                else if ( columnNames.size() ) {
-                    knn = columnNames.size() + 1;
-                }
-                
-                if ( verbose ) {
-                    std::stringstream msg;
-                    msg << "Parameters::Validate(): Set knn = " << knn
-                        << " for Simplex (embedded = true). " << std::endl;
-                    std::cout << msg.str();
+            else if ( columnNames.size() ) {
+                if ( E != (int) columnNames.size() ) {
+                    E = columnNames.size();
                 }
             }
         }
+
+        if ( knn < 1 ) {
+            knn = E + 1;
+
+            if ( verbose ) {
+                std::stringstream msg;
+                msg << "Parameters::Validate(): Set knn = " << knn
+                    << " (E+1) for Simplex. " << std::endl;
+                std::cout << msg.str();
+            }
+        }
+
         if ( knn < E + 1 ) {
             std::stringstream errMsg;
             errMsg << "Parameters::Validate(): Simplex knn of " << knn
@@ -492,6 +572,15 @@ void Parameters::Validate() {
         throw std::runtime_error( "Parameters::Validate() "
                                   "Prediction method error.\n" );
     }
+
+    if ( method == Method::Simplex or method == Method::SMap ) {
+        if ( E < 1 ) {
+            std::stringstream errMsg;
+            errMsg << "Parameters::Validate() E = " << E
+                   << " is invalid with embedded = true.\n" ;
+            throw std::runtime_error( errMsg.str() );
+        }
+    }
 }
 
 //------------------------------------------------------------
@@ -535,6 +624,17 @@ void Parameters::DeleteLibPred() {
         }
     }
 
+    bool deleteDisjointLibIndex = false;
+    for ( auto element  = deleted_lib_elements.begin();
+               element != deleted_lib_elements.end(); element++ ) {
+        it = std::find( disjointLibraryRows.begin(),
+                        disjointLibraryRows.end(), *element );
+        if ( it != prediction.end() ) {
+            deleteDisjointLibIndex = true;
+            break;
+        }
+    }
+
     bool deletePredIndex = false;
     for ( auto element  = deleted_pred_elements.begin();
                element != deleted_pred_elements.end(); element++ ) {
@@ -563,7 +663,7 @@ void Parameters::DeleteLibPred() {
     // Erase elements of row indices that were deleted
     if ( deleteLibIndex ) {
         for ( auto element  = deleted_lib_elements.begin();
-              element != deleted_lib_elements.end(); element++ ) {
+                   element != deleted_lib_elements.end(); element++ ) {
 
             std::vector< size_t >::iterator it;
             it = std::find( library.begin(), library.end(), *element );
@@ -574,6 +674,20 @@ void Parameters::DeleteLibPred() {
         }
     }
 
+    if ( deleteDisjointLibIndex ) {
+        for ( auto element  = deleted_lib_elements.begin();
+                   element != deleted_lib_elements.end(); element++ ) {
+
+            std::vector< size_t >::iterator it;
+            it = std::find( disjointLibraryRows.begin(),
+                            disjointLibraryRows.end(), *element );
+            
+            if ( it != disjointLibraryRows.end() ) {
+                disjointLibraryRows.erase( it );
+            }
+        }
+    }
+
     if ( deletePredIndex ) {
         for ( auto element  = deleted_pred_elements.begin();
               element != deleted_pred_elements.end(); element++ ) {
@@ -595,6 +709,10 @@ void Parameters::DeleteLibPred() {
         for ( auto li = library.begin(); li != library.end(); li++ ) {
             *li = *li - shift;
         }
+        for ( auto li  = disjointLibraryRows.begin();
+                   li != disjointLibraryRows.end(); li++ ) {
+            *li = *li - shift;
+        }
         for ( auto pi = prediction.begin(); pi != prediction.end(); pi++ ) {
             *pi = *pi - shift;
         }
